import os
import re
import time
import sys
from hashlib import sha256

from twisted.internet import defer
from twisted.trial.runner import filenameToModule
from twisted.python import usage, reflect

from ooni import __version__ as ooniprobe_version
from ooni import otime
from ooni.tasks import Measurement
from ooni.utils import log, sanitize_options, randomStr
from ooni.utils.net import hasRawSocketPermission
from ooni.settings import config

from ooni import errors as e

from inspect import getmembers
from StringIO import StringIO


class NoTestCasesFound(Exception):
    pass


def getTestClassFromFile(net_test_file):
    """
    Will return the first class that is an instance of NetTestCase.

    XXX this means that if inside of a test there are more than 1 test case
        then we will only run the first one.
    """
    module = filenameToModule(net_test_file)
    for __, item in getmembers(module):
        try:
            assert issubclass(item, NetTestCase)
            return item
        except (TypeError, AssertionError):
            pass


def getOption(opt_parameter, required_options, type='text'):
    """
    Arguments:
        usage_options: a list as should be the optParameters of an UsageOptions
            class.

        required_options: a list containing the strings of the options that are
            required.

        type: a string containing the type of the option.

    Returns:
        a dict containing
            {
                'description': the description of the option,
                'default': the default value of the option,
                'required': True|False if the option is required or not,
                'type': the type of the option ('text' or 'file')
            }
    """
    option_name, _, default, description = opt_parameter
    if option_name in required_options:
        required = True
    else:
        required = False

    return {'description': description,
            'value': default, 'required': required,
            'type': type
            }


def getArguments(test_class):
    arguments = {}
    if test_class.inputFile:
        option_name = test_class.inputFile[0]
        arguments[option_name] = getOption(
            test_class.inputFile,
            test_class.requiredOptions,
            type='file')
    try:
        list(test_class.usageOptions.optParameters)
    except AttributeError:
        return arguments

    for opt_parameter in test_class.usageOptions.optParameters:
        option_name = opt_parameter[0]
        opt_type = "text"
        if opt_parameter[3].lower().startswith("file"):
            opt_type = "file"
        arguments[option_name] = getOption(
            opt_parameter,
            test_class.requiredOptions,
            type=opt_type)

    return arguments


def test_class_name_to_name(test_class_name):
    return test_class_name.lower().replace(' ', '_')


def getNetTestInformation(net_test_file):
    """
    Returns a dict containing:

    {
        'id': the test filename excluding the .py extension,
        'name': the full name of the test,
        'description': the description of the test,
        'version': version number of this test,
        'arguments': a dict containing as keys the supported arguments and as
            values the argument description.
    }
    """
    test_class = getTestClassFromFile(net_test_file)

    test_id = os.path.basename(net_test_file).replace('.py', '')
    information = {'id': test_id,
                   'name': test_class.name,
                   'description': test_class.description,
                   'version': test_class.version,
                   'arguments': getArguments(test_class),
                   'path': net_test_file,
                   }
    return information


def usageOptionsFactory(test_name, test_version):

    class UsageOptions(usage.Options):
        optParameters = []
        optFlags = []

        synopsis = "{} {} [options]".format(
            os.path.basename(sys.argv[0]),
            test_name
        )

        def opt_version(self):
            """
            Display the net_test version and exit.
            """
            print "{} version: {}".format(test_name, test_version)
            sys.exit(0)

    return UsageOptions

def netTestCaseFactory(test_class, local_options):
    class NetTestCaseWithLocalOptions(test_class):
        localOptions = local_options
    return NetTestCaseWithLocalOptions

ONION_INPUT_REGEXP = re.compile("(httpo://[a-z0-9]{16}\.onion)/input/(["
                                "a-z0-9]{64})$")

class NetTestLoader(object):
    method_prefix = 'test'
    collector = None
    yamloo = True
    requiresTor = False

    def __init__(self, options, test_file=None, test_string=None,
                 annotations={}):
        self.options = options
        self.annotations = annotations

        self.requiresTor = False

        self.testName = ""
        self.testVersion = ""
        self.reportId = None

        self.testHelpers = {}
        self.missingTestHelpers = []
        self.usageOptions = None
        self.inputFiles = []

        self._testCases = []
        self.localOptions = None

        if test_file:
            self.loadNetTestFile(test_file)
        elif test_string:
            self.loadNetTestString(test_string)

    def getTestDetails(self):
        return {
            'probe_asn': config.probe_ip.geodata['asn'],
            'probe_cc': config.probe_ip.geodata['countrycode'],
            'probe_ip': config.probe_ip.geodata['ip'],
            'probe_city': config.probe_ip.geodata['city'],
            'software_name': 'ooniprobe',
            'software_version': ooniprobe_version,
            'options': sanitize_options(self.options),
            'annotations': self.annotations,
            'data_format_version': '0.2.0',
            'test_name': self.testName,
            'test_version': self.testVersion,
            'test_helpers': self.testHelpers,
            'test_start_time': otime.timestampNowLongUTC(),
            'input_hashes': [input_file['hash']
                             for input_file in self.inputFiles],
            'report_id': self.reportId
        }

    def getTestCases(self):
        """
        Specialises the test_classes to include the local options.
        :return:
        """
        test_cases = []
        for test_class, test_method in self._testCases:
            test_cases.append((netTestCaseFactory(test_class,
                                                  self.localOptions),
                               test_method))
        return test_cases

    def _accumulateInputFiles(self, test_class):
        if not test_class.inputFile:
            return

        key = test_class.inputFile[0]
        filename = self.localOptions[key]
        if not filename:
            return

        input_file = {
            'key': key,
            'test_options': self.localOptions,
            'hash': None,

            'url': None,
            'address': None,

            'filename': None
        }
        m = ONION_INPUT_REGEXP.match(filename)
        if m:
            input_file['url'] = filename
            input_file['address'] = m.group(1)
            input_file['hash'] = m.group(2)
        else:
            input_file['filename'] = filename
            try:
                with open(filename) as f:
                    h = sha256()
                    for l in f:
                        h.update(l)
            except:
                raise e.InvalidInputFile(filename)
            input_file['hash'] = h.hexdigest()
        self.inputFiles.append(input_file)

    def _accumulateTestOptions(self, test_class):
        """
        Accumulate the optParameters and optFlags for the NetTestCase class
        into the usageOptions of the NetTestLoader.
        """
        if getattr(test_class.usageOptions, 'optParameters', None):
            for parameter in test_class.usageOptions.optParameters:
                # XXX should look into if this is still necessary, seems like
                # something left over from a bug in some nettest.
                # In theory optParameters should always have a length of 4.
                if len(parameter) == 5:
                    parameter.pop()
                self.usageOptions.optParameters.append(parameter)

        if getattr(test_class, 'inputFile', None):
            self.usageOptions.optParameters.append(test_class.inputFile)

        if getattr(test_class, 'baseParameters', None):
            for parameter in test_class.baseParameters:
                self.usageOptions.optParameters.append(parameter)

        if getattr(test_class, 'baseFlags', None):
            for flag in test_class.baseFlags:
                self.usageOptions.optFlags.append(flag)

    def parseLocalOptions(self):
        """
        Parses the localOptions for the NetTestLoader.
        """
        self.localOptions = self.usageOptions()
        try:
            self.localOptions.parseOptions(self.options)
        except usage.UsageError:
            tb = sys.exc_info()[2]
            raise e.OONIUsageError(self), None, tb

    def _checkTestClassOptions(self, test_class):
        if test_class.requiresRoot and not hasRawSocketPermission():
            raise e.InsufficientPrivileges
        if test_class.requiresTor:
            self.requiresTor = True
        self._checkRequiredOptions(test_class)
        self._setTestHelpers(test_class)
        test_instance = netTestCaseFactory(test_class, self.localOptions)()
        test_instance.requirements()

    def _setTestHelpers(self, test_class):
        for option, name in test_class.requiredTestHelpers.items():
            if self.localOptions.get(option, None):
                self.testHelpers[option] = self.localOptions[option]

    def _checkRequiredOptions(self, test_class):
        missing_options = []
        for required_option in test_class.requiredOptions:
            log.debug("Checking if %s is present" % required_option)
            if required_option not in self.localOptions or \
                    self.localOptions[required_option] is None:
                missing_options.append(required_option)
        missing_test_helpers = [opt in test_class.requiredTestHelpers.keys()
                                for opt in missing_options]
        if len(missing_test_helpers) and all(missing_test_helpers):
            self.missingTestHelpers = map(lambda x:
                                            (x, test_class.requiredTestHelpers[x]),
                                          missing_options)
            raise e.MissingTestHelper(missing_options, test_class)
        elif missing_options:
            raise e.MissingRequiredOption(missing_options, test_class)

    def loadNetTestString(self, net_test_string):
        """
        Load NetTest from a string.
        WARNING input to this function *MUST* be sanitized and *NEVER* take
        untrusted input.
        Failure to do so will result in code exec.

        net_test_string:

            a string that contains the net test to be run.
        """
        net_test_file_object = StringIO(net_test_string)

        ns = {}
        test_cases = []
        exec net_test_file_object.read() in ns
        for item in ns.itervalues():
            test_cases.extend(self._getTestMethods(item))

        if not test_cases:
            raise e.NoTestCasesFound

        self._setupTestCases(test_cases)

    def loadNetTestFile(self, net_test_file):
        """
        Load NetTest from a file.
        """
        test_cases = []
        module = filenameToModule(net_test_file)
        for __, item in getmembers(module):
            test_cases.extend(self._getTestMethods(item))

        if not test_cases:
            raise e.NoTestCasesFound

        self._setupTestCases(test_cases)

    def _setupTestCases(self, test_cases):
        """
        Creates all the necessary test_cases (a list of tuples containing the
        NetTestCase (test_class, test_method))

        example:
            [(test_classA, [test_method1,
                            test_method2,
                            test_method3,
                            test_method4,
                            test_method5]),
            (test_classB, [test_method1,
                           test_method2])]

        Note: the inputs must be valid for test_classA and test_classB.

        net_test_file:
            is either a file path or a file like object that will be used to
            generate the test_cases.
        """
        test_class, _ = test_cases[0]
        self.testName = test_class_name_to_name(test_class.name)
        self.testVersion = test_class.version
        self._testCases = test_cases

        self.usageOptions = usageOptionsFactory(self.testName,
                                                self.testVersion)

        if config.reports.unique_id is True:
            self.reportId = randomStr(64)

        for test_class, test_methods in self._testCases:
            self._accumulateTestOptions(test_class)

    def checkOptions(self):
        self.parseLocalOptions()
        test_options_exc = None
        usage_options = self._testCases[0][0].usageOptions
        for test_class, test_methods in self._testCases:
            try:
                self._accumulateInputFiles(test_class)
                self._checkTestClassOptions(test_class)
                if usage_options != test_class.usageOptions:
                    raise e.IncoherentOptions(usage_options.__name__,
                                              test_class.usageOptions.__name__)
            except Exception as exc:
                test_options_exc = exc

        if test_options_exc is not None:
            raise test_options_exc

    def _getTestMethods(self, item):
        """
        Look for test_ methods in subclasses of NetTestCase
        """
        test_cases = []
        try:
            assert issubclass(item, NetTestCase)
            methods = reflect.prefixedMethodNames(item, self.method_prefix)
            test_methods = []
            for method in methods:
                test_methods.append(self.method_prefix + method)
            if test_methods:
                test_cases.append((item, test_methods))
        except (TypeError, AssertionError):
            pass
        return test_cases


class NetTestState(object):

    def __init__(self, allTasksDone):
        """
        This keeps track of the state of a running NetTests case.

        Args:
            allTasksDone is a deferred that will get fired once all the NetTest
            cases have reached a final done state.
        """
        self.doneTasks = 0
        self.tasks = 0

        self.completedScheduling = False
        self.allTasksDone = allTasksDone

    def taskCreated(self):
        self.tasks += 1

    def checkAllTasksDone(self):
        log.debug("Checking all tasks for completion %s == %s" %
                  (self.doneTasks, self.tasks))
        if self.completedScheduling and \
                self.doneTasks == self.tasks:
            self.allTasksDone.callback(self.doneTasks)

    def taskDone(self):
        """
        This is called every time a task has finished running.
        """
        self.doneTasks += 1
        self.checkAllTasksDone()

    def allTasksScheduled(self):
        """
        This should be called once all the tasks that need to run have been
        scheduled.

        XXX this is ghetto.
        The reason for which we are calling allTasksDone inside of the
        allTasksScheduled method is called after all tasks are done, then we
        will run into a race condition. The race is that we don't end up
        checking that all the tasks are complete because no task is to be
        scheduled.
        """
        self.completedScheduling = True
        self.checkAllTasksDone()


class NetTest(object):
    director = None

    def __init__(self, test_cases, test_details, report):
        """
        net_test_loader:
             an instance of :class:ooni.nettest.NetTestLoader containing
             the test to be run.

        report:
            an instance of :class:ooni.reporter.Reporter
        """
        self.report = report

        self.testDetails = test_details
        self.testCases = test_cases

        self.summary = {}

        # This will fire when all the measurements have been completed and
        # all the reports are done. Done means that they have either completed
        # successfully or all the possible retries have been reached.
        self.done = defer.Deferred()
        self.done.addCallback(self.doneNetTest)

        self.state = NetTestState(self.done)

    def __str__(self):
        return ' '.join(tc.name for tc, _ in self.testCases)

    def uniqueClasses(self):
        classes = []
        for test_class, test_method in self.testCases:
            if test_class not in classes:
                classes.append(test_class)
        return classes

    def doneNetTest(self, result):
        if self.summary:
            print "Summary for %s" % self.testDetails['test_name']
            print "------------" + "-"*len(self.testDetails['test_name'])
            for test_class in self.uniqueClasses():
                test_instance = test_class()
                test_instance.displaySummary(self.summary)
        if self.testDetails["report_id"]:
            print "Report ID: %s" % self.testDetails["report_id"]

    def doneReport(self, report_results):
        """
        This will get called every time a report is done and therefore a
        measurement is done.

        The state for the NetTest is informed of the fact that another task has
        reached the done state.
        """
        self.state.taskDone()

        return report_results

    def makeMeasurement(self, test_instance, test_method, test_input=None):
        """
        Creates a new instance of :class:ooni.tasks.Measurement and add's it's
        callbacks and errbacks.

        Args:
            test_class:
                a subclass of :class:ooni.nettest.NetTestCase

            test_method:
                a string that represents the method to be called on test_class

            test_input:
                optional argument that represents the input to be passed to the
                NetTestCase

        """
        measurement = Measurement(test_instance, test_method, test_input)
        measurement.netTest = self

        if self.director:
            measurement.done.addCallback(self.director.measurementSucceeded,
                                         measurement)
            measurement.done.addErrback(self.director.measurementFailed,
                                        measurement)
        return measurement

    @defer.inlineCallbacks
    def initialize(self):
        for test_class, _ in self.testCases:
            # Initialize Input Processor
            test_class.inputs = yield defer.maybeDeferred(
                test_class().getInputProcessor
            )

            # Run the setupClass method
            yield defer.maybeDeferred(
                test_class.setUpClass
            )

    def generateMeasurements(self):
        """
        This is a generator that yields measurements and registers the
        callbacks for when a measurement is successful or has failed.
        """

        for test_class, test_methods in self.testCases:
            # load a singular input processor for all instances
            all_inputs = test_class.inputs
            for test_input in all_inputs:
                measurements = []
                test_instance = test_class()
                # Set each instances inputs to a singular input processor
                test_instance.inputs = all_inputs
                test_instance._setUp()
                test_instance.summary = self.summary
                for method in test_methods:
                    log.debug("Running %s %s" % (test_instance, method))
                    measurement = self.makeMeasurement(
                        test_instance,
                        method,
                        test_input)
                    measurements.append(measurement.done)
                    self.state.taskCreated()
                    yield measurement

                # When the measurement.done callbacks have all fired
                # call the postProcessor before writing the report
                if self.report:
                    post = defer.DeferredList(measurements)

                    @post.addBoth
                    def set_runtime(results):
                        runtime = time.time() - test_instance._start_time
                        for _, m in results:
                            m.testInstance.report['test_runtime'] = runtime
                        test_instance.report['test_runtime'] = runtime
                        return results

                    # Call the postProcessor, which must return a single report
                    # or a deferred
                    post.addCallback(test_instance.postProcessor)

                    def noPostProcessor(failure, report):
                        failure.trap(e.NoPostProcessor)
                        return report
                    post.addErrback(noPostProcessor, test_instance.report)
                    post.addCallback(self.report.write)

                if self.report and self.director:
                    # ghetto hax to keep NetTestState counts are accurate
                    [post.addBoth(self.doneReport) for _ in measurements]

        self.state.allTasksScheduled()


class NetTestCase(object):

    """
    This is the base of the OONI nettest universe. When you write a nettest
    you will subclass this object.

    * inputs: can be set to a static set of inputs. All the tests (the methods
      starting with the "test" prefix) will be run once per input. At every
      run the _input_ attribute of the TestCase instance will be set to the
      value of the current iteration over inputs.  Any python iterable object
      can be set to inputs.

    * inputFile: attribute should be set to an array containing the command
      line argument that should be used as the input file. Such array looks
      like this:

          ``["commandlinearg", "c", "default value" "The description"]``

      The second value of such arrray is the shorthand for the command line
      arg. The user will then be able to specify inputs to the test via:

          ``ooniprobe mytest.py --commandlinearg path/to/file.txt``

      or

          ``ooniprobe mytest.py -c path/to/file.txt``


    * inputProcessor: should be set to a function that takes as argument a
      filename and it will return the input to be passed to the test
      instance.

    * name: should be set to the name of the test.

    * author: should contain the name and contact details for the test author.
      The format for such string is as follows:

          ``The Name <email@example.com>``

    * version: is the version string of the test.

    * requiresRoot: set to True if the test must be run as root.

    * usageOptions: a subclass of twisted.python.usage.Options for processing
        of command line arguments

    * localOptions: contains the parsed command line arguments.

    Quirks:
    Every class that is prefixed with test *must* return a
    twisted.internet.defer.Deferred.
    """
    name = "This test is nameless"
    author = "Jane Doe <foo@example.com>"
    version = "0.0.0"
    description = "Sorry, this test has no description :("

    inputs = None
    inputFile = None
    inputFilename = None

    usageOptions = usage.Options

    optParameters = None
    baseParameters = None
    baseFlags = None

    requiredTestHelpers = {}
    requiredOptions = []
    requiresRoot = False
    requiresTor = False

    localOptions = {}

    @classmethod
    def setUpClass(cls):
        """
        You can override this hook with logic that should be run once before
        any test method in the NetTestCase is run.
        This can be useful to populate class attribute that should be valid
        for all the runtime of the NetTest.
        """
        pass

    def _setUp(self):
        """
        This is the internal setup method to be overwritten by templates.
        It gets called once for every input.
        """
        self.report = {}

    def requirements(self):
        """
        Place in here logic that will be executed before the test is to be run.
        If some condition is not met then you should raise an exception.
        """
        pass

    def setUp(self):
        """
        Place here your logic to be executed when the test is being setup.
        It gets called once every test method + input.
        """
        pass

    def postProcessor(self, measurements):
        """
        Subclass this to do post processing tasks that are to occur once all
        the test methods have been called once per input.
        postProcessing works exactly like test methods, in the sense that
        anything that gets written to the object self.report[] will be added to
        the final test report.
        You should also place in this method any logic that is required for
        generating the summary.
        """
        raise e.NoPostProcessor

    def displaySummary(self, summary):
        """
        This gets called after the test has run to allow printing out of a
        summary of the test run.
        """
        pass

    def inputProcessor(self, filename):
        """
        You may replace this with your own custom input processor. It takes as
        input a file name.

        An inputProcessor is an iterator that will yield one item from the file
        and takes as argument a filename.

        This can be useful when you have some input data that is in a certain
        format and you want to set the input attribute of the test to something
        that you will be able to properly process.

        For example you may wish to have an input processor that will allow you
        to ignore comments in files. This can be easily achieved like so::

            fp = open(filename)
            for x in fp.xreadlines():
                if x.startswith("#"):
                    continue
                yield x.strip()
            fp.close()

        Other fun stuff is also possible.
        """
        log.debug("Running default input processor")
        with open(filename) as f:
            for line in f:
                l = line.strip()
                # Skip empty lines
                if not l:
                    continue
                # Skip comment lines
                elif l.startswith('#'):
                    continue
                yield l

    @property
    def inputFileSpecified(self):
        """
        Returns:
            True
                when inputFile is supported and is specified
            False
                when input is either not support or not specified
        """
        if not self.inputFile:
            return False

        k = self.inputFile[0]
        if self.localOptions.get(k):
            return True
        else:
            return False

    def getInputProcessor(self):
        """
        This method must be called after all options are validated by
        _checkValidOptions and _checkRequiredOptions, which ensure that
        if the inputFile is a required option it will be present.

        We check to see if it's possible to have an input file and if the user
        has specified such file.


        If the operations to be done here are network related or blocking, they
        should be wrapped in a deferred. That is the return value of this
        method should be a :class:`twisted.internet.defer.Deferred`.

        Returns:
            a generator that will yield one item from the file based on the
            inputProcessor.
        """
        if self.inputFileSpecified:
            self.inputFilename = self.localOptions[self.inputFile[0]]
            return self.inputProcessor(self.inputFilename)

        if self.inputs:
            return self.inputs

        return [None]

    def __repr__(self):
        return "<%s inputs=%s>" % (self.__class__, self.inputs)



"""
ooniprobe: a network interference detection tool
================================================

.. image:: https://travis-ci.org/TheTorProject/ooni-probe.png?branch=master
    :target: https://travis-ci.org/TheTorProject/ooni-probe

.. image:: https://coveralls.io/repos/TheTorProject/ooni-probe/badge.png
    :target: https://coveralls.io/r/TheTorProject/ooni-probe

___________________________________________________________________________

.. image:: https://ooni.torproject.org/images/ooni-header-mascot.png
    :target: https:://ooni.torproject.org/

OONI, the Open Observatory of Network Interference, is a global observation
network which aims is to collect high quality data using open methodologies,
using Free and Open Source Software (FL/OSS) to share observations and data
about the various types, methods, and amounts of network tampering in the
world.

Read this before running ooniprobe!
-----------------------------------

Running ooniprobe is a potentially risky activity. This greatly depends on the
jurisdiction in which you are in and which test you are running. It is
technically possible for a person observing your internet connection to be
aware of the fact that you are running ooniprobe. This means that if running
network measurement tests is something considered to be illegal in your country
then you could be spotted.

Furthermore, ooniprobe takes no precautions to protect the install target machine
from forensics analysis. If the fact that you have installed or used ooni
probe is a liability for you, please be aware of this risk.


Setup ooniprobe
-------------------

To install ooniprobe you will need the following dependencies:

    * python
    * python-dev
    * python-setuptools
    * build-essential
    * libdumbnet1
    * python-dumbnet
    * python-libpcap
    * tor
    * libgeoip-dev
    * libpcap0.8-dev
    * libssl-dev
    * libffi-dev
    * libdumbnet-dev


On debian based systems this can generally be done by running:

.. code:: bash

    sudo apt-get install -y build-essential libdumbnet-dev libpcap-dev libgeoip-dev libffi-dev python-dev python-pip

When you got them run:

.. code:: bash

    sudo pip install ooniprobe

Using ooniprobe
---------------

To generate a test deck for your country, cd to the directory where you want it
and run:

.. code:: bash

    oonideckgen


To setup a daily cronjob run this:

.. code:: bash

    (crontab -l 2>/dev/null; echo "@daily ooniprobe `oonideckgen | grep -e '^ooniprobe'`") | crontab -

Have fun!
"""

from __future__ import print_function

from ooni import __version__, __author__
import os
import sys
import shutil
import tempfile
import subprocess
from ConfigParser import SafeConfigParser

from os.path import join as pj
from setuptools import setup, Command
from setuptools.command.install import install
from distutils.spawn import find_executable

GEOIP_ASN_URL = "https://download.maxmind.com/download/geoip/database/asnum/GeoIPASNum.dat.gz"
GEOIP_URL = "https://geolite.maxmind.com/download/geoip/database/GeoLiteCountry/GeoIP.dat.gz"
TEST_LISTS_URL = "https://github.com/citizenlab/test-lists/archive/master.zip"

def run_command(args, cwd=None):
    try:
        p = subprocess.Popen(args, stdout=subprocess.PIPE, cwd=cwd)
    except EnvironmentError:
        return None
    stdout = p.communicate()[0].strip()
    if p.returncode != 0:
        return None
    return stdout

def is_lepidopter():
    if os.path.exists("/etc/default/lepidopter"):
        return True
    return False

class OoniInstall(install):
    def gen_config(self, share_path):
        config_file = pj(tempfile.mkdtemp(), "ooniprobe.conf.sample")
        o = open(config_file, "w+")
        with open("data/ooniprobe.conf.sample") as f:
            for line in f:
                if "/usr/share" in line:
                    line = line.replace("/usr/share", share_path)
                o.write(line)
        o.close()
        return config_file

    def set_data_files(self, prefix):
        share_path = pj(prefix, 'share')
        if prefix.startswith("/usr"):
            var_path = "/var/lib/"
        else:
            var_path = pj(prefix, 'var', 'lib')

        for root, dirs, file_names in os.walk('data/'):
            files = []
            for file_name in file_names:
                if file_name.endswith('.pyc'):
                    continue
                elif file_name.endswith('.dat') and \
                        file_name.startswith('Geo'):
                    continue
                elif file_name == "ooniprobe.conf.sample":
                    files.append(self.gen_config(share_path))
                    continue
                files.append(pj(root, file_name))
            self.distribution.data_files.append(
                [
                    pj(share_path, 'ooni', root.replace('data/', '')),
                    files
                ]
            )
        settings = SafeConfigParser()
        settings.add_section("directories")
        settings.set("directories", "usr_share",
                     os.path.join(share_path, "ooni"))
        settings.set("directories", "var_lib",
                     os.path.join(var_path, "ooni"))
        with open("ooni/settings.ini", "w+") as fp:
            settings.write(fp)

        try:
            os.makedirs(pj(var_path, 'ooni'))
        except OSError:
            pass
        try:
            os.makedirs(pj(share_path, 'ooni'))
        except OSError:
            pass

    def ooniresources(self):
        ooniresources = find_executable("ooniresources")
        process = subprocess.Popen([ooniresources],
                                   stdout=sys.stdout.fileno(),
                                   stderr=sys.stderr.fileno())
        process.wait()

    def update_lepidopter_config(self):
        try:
            shutil.copyfile("data/configs/lepidopter-ooniprobe.conf",
                            "/etc/ooniprobe/ooniprobe.conf")
            shutil.copyfile("data/configs/lepidopter-oonireport.conf",
                            "/etc/ooniprobe/oonireport.conf")
        except Exception:
            print("ERR: Failed to copy configuration files to /etc/ooniprobe/")

    def run(self):
        prefix = os.path.abspath(self.prefix)
        self.set_data_files(prefix)
        self.do_egg_install()
        self.ooniresources()
        if is_lepidopter():
            self.update_lepidopter_config()

class ExecutableNotFound(Exception):
    pass

class CreateOoniResources(Command):
    description = ("Create ooni-resources.tar.gz containing test-lists and "
                   "GeoIP data files")
    user_options = []

    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def download(self, url, directory, filename):
        dst_path = pj(directory, filename)
        args = [
            self.wget,
            "-O",
            dst_path,
            url
        ]
        out = run_command(args)
        if out is None:
            raise Exception("Failed to download {0}".format(url))
        return dst_path

    def find_executables(self):
        self.wget = find_executable("wget")
        if not self.wget:
            raise ExecutableNotFound("wget")
        self.tar = find_executable("tar")
        if not self.tar:
            raise ExecutableNotFound("tar")
        self.unzip = find_executable("unzip")
        if not self.unzip:
            raise ExecutableNotFound("unzip")
        self.gunzip = find_executable("gunzip")
        if not self.gunzip:
            raise ExecutableNotFound("gunzip")

    def run(self):
        dst_path = "dist/ooni-resources.tar.gz"

        try:
            self.find_executables()
        except ExecutableNotFound as enf:
            print("ERR: Could not find '{0}'".format(enf.message))
            return

        tmp_dir = tempfile.mkdtemp()
        pkg_dir = tempfile.mkdtemp()

        os.mkdir(pj(pkg_dir, "resources"))
        os.mkdir(pj(pkg_dir, "GeoIP"))

        try:
            geoip_asn_path = self.download(GEOIP_ASN_URL, tmp_dir, "GeoIPASNum.dat.gz")
        except Exception as exc:
            print(exc.message)
            return
        try:
            geoip_path = self.download(GEOIP_URL, tmp_dir, "GeoIP.dat.gz")
        except Exception as exc:
            print(exc.message)
            return
        try:
            test_lists_path = self.download(TEST_LISTS_URL, tmp_dir, "master.zip")
        except Exception as exc:
            print(exc.message)
            return

        run_command([self.gunzip, geoip_asn_path])
        run_command([self.gunzip, geoip_path])
        run_command([self.unzip, "-d", tmp_dir, test_lists_path])

        shutil.move(pj(tmp_dir, "GeoIP.dat"),
                    pj(pkg_dir, "GeoIP", "GeoIP.dat"))
        shutil.move(pj(tmp_dir, "GeoIPASNum.dat"),
                    pj(pkg_dir, "GeoIP", "GeoIPASNum.dat"))
        shutil.move(pj(tmp_dir, "test-lists-master", "lists"),
                    pj(pkg_dir, "resources", "citizenlab-test-lists"))
        # Don't include services and official lists
        shutil.rmtree(
            pj(pkg_dir,
               "resources",
               "citizenlab-test-lists",
               "services"),
            ignore_errors=True)
        shutil.rmtree(
            pj(pkg_dir,
               "resources",
               "citizenlab-test-lists",
               "official"),
            ignore_errors=True)
        run_command([self.tar, "cvzf", dst_path, "-C", pkg_dir, "."])

        # Cleanup
        shutil.rmtree(pkg_dir, ignore_errors=True)
        shutil.rmtree(tmp_dir, ignore_errors=True)
        print("Written ooniresources to {0}".format(dst_path))


install_requires = []
dependency_links = []
data_files = []
packages = [
    'ooni',
    'ooni.api',
    'ooni.common',
    'ooni.deckgen',
    'ooni.deckgen.processors',
    'ooni.kit',
    'ooni.nettests',
    'ooni.nettests.manipulation',
    'ooni.nettests.experimental',
    'ooni.nettests.scanning',
    'ooni.nettests.blocking',
    'ooni.nettests.third_party',
    'ooni.report',
    'ooni.resources',
    'ooni.templates',
    'ooni.tests',
    'ooni.utils'
]

with open('requirements.txt') as f:
    for line in f:
        if line.startswith("#"):
            continue
        if line.startswith('https'):
            dependency_links.append(line)
            continue
        install_requires.append(line)

setup(
    name="ooniprobe",
    version=__version__,
    author=__author__,
    author_email="contact@openobservatory.org",
    description="Network measurement tool for"
                "identifying traffic manipulation and blocking.",
    long_description=__doc__,
    license='BSD 2 clause',
    url="https://ooni.torproject.org/",
    package_dir={'ooni': 'ooni'},
    data_files=data_files,
    packages=packages,
    include_package_data=True,
    scripts=["bin/oonideckgen", "bin/ooniprobe",
             "bin/oonireport", "bin/ooniresources"],
    dependency_links=dependency_links,
    install_requires=install_requires,
    zip_safe=False,
    cmdclass={
        "install": OoniInstall,
        "create_ooniresources": CreateOoniResources
    },
    classifiers=(
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Framework :: Twisted",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Telecommunications Industry",
        "License :: OSI Approved :: BSD License"
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2 :: Only",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: BSD",
        "Operating System :: POSIX :: BSD :: BSD/OS",
        "Operating System :: POSIX :: BSD :: FreeBSD",
        "Operating System :: POSIX :: BSD :: NetBSD",
        "Operating System :: POSIX :: BSD :: OpenBSD",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Unix",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Security",
        "Topic :: Security :: Cryptography",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Testing :: Traffic Generation",
        "Topic :: System :: Networking :: Monitoring",
    )
)



from twisted.trial import unittest

from ooni.ui.web.server import AppRouting

class DummyRequest(object):
    def __init__(self, val):
        self.val = val

    def __eq__(self, other):
        return self.val == other.val

class TestServer(unittest.TestCase):
    def test_app_routing(self):
        calls = []
        class DummyApp(object):
            app = AppRouting()

            @app.route("^/foo/(.*)$")
            def endpoint_a(self, request, value):
                calls.append((self, request, value))
                return "foo" + value

        da = DummyApp()
        self.assertEqual(da.app.execute("/foo/value", "GET", DummyRequest(1)),
                         "foovalue")
        self.assertEqual(calls, [(da, DummyRequest(1), "value")])

    def test_app_routing_with_prefix(self):
        calls = []
        class DummyApp(object):
            app = AppRouting(prefix="/api")

            @app.route("^/foo/(.*)$")
            def endpoint_a(self, request, value):
                calls.append((self, request, value))
                return "foo" + value

        da = DummyApp()
        self.assertEqual(da.app.execute("/api/foo/value", "GET",
                                        DummyRequest(1)),
                         "foovalue")
        self.assertEqual(calls, [(da, DummyRequest(1), "value")])



from twisted.internet import protocol, defer, reactor

from ooni.settings import config
from ooni.nettest import NetTestCase
from ooni.utils import log


class ProcessDirector(protocol.ProcessProtocol):
    def __init__(self, d, finished=None, timeout=None, stdin=None):
        self.d = d
        self.stderr = ""
        self.stdout = ""
        self.finished = finished
        self.timeout = timeout
        self.stdin = stdin

        self.timer = None
        self.exit_reason = None

    def cancelTimer(self):
        if self.timeout and self.timer:
            self.timer.cancel()
            self.timer = None

    def close(self, reason=None):
        self.reason = reason
        self.transport.loseConnection()

    def resetTimer(self):
        if self.timeout is not None:
            if self.timer is not None and self.timer.active():
                self.timer.cancel()
            self.timer = reactor.callLater(self.timeout,
                                           self.close,
                                           "timeout_reached")

    def finish(self, exit_reason=None):
        if not self.exit_reason:
            self.exit_reason = exit_reason
        data = {
            "stderr": self.stderr,
            "stdout": self.stdout,
            "exit_reason": self.exit_reason
        }
        self.d.callback(data)

    def shouldClose(self):
        if self.finished is None:
            return False
        return self.finished(self.stdout, self.stderr)

    def connectionMade(self):
        self.resetTimer()
        if self.stdin is not None:
            self.transport.write(self.stin)
            self.transport.closeStdin()

    def outReceived(self, data):
        log.debug("STDOUT: %s" % data)
        self.stdout += data
        if self.shouldClose():
            self.close("condition_met")
        self.handleRead(data,  None)

    def errReceived(self, data):
        log.debug("STDERR: %s" % data)
        self.stderr += data
        if self.shouldClose():
            self.close("condition_met")
        self.handleRead(None,  data)


    def inConnectionLost(self):
        log.debug("inConnectionLost")
        # self.d.callback(self.data())

    def outConnectionLost(self):
        log.debug("outConnectionLost")

    def errConnectionLost(self):
        log.debug("errConnectionLost")

    def processExited(self, reason):
        log.debug("Exited %s" % reason)

    def processEnded(self, reason):
        log.debug("Ended %s" % reason)
        self.finish("process_done")

    def handleRead(self,  stdout,  stderr=None):
        pass


class ProcessTest(NetTestCase):
    name = "Base Process Test"
    version = "0.1"

    requiresRoot = False
    timeout = 5
    processDirector = None

    def _setUp(self):
        super(ProcessTest, self)._setUp()

    def processEnded(self, result, command):
        log.debug("Finished %s: %s" % (command, result))
        if not isinstance(self.report.get('commands'), list):
            self.report['commands'] = []

        # Attempt to redact the IP address of the probe from the standard output
        if config.privacy.includeip is False and config.probe_ip.address is not None:
            result['stdout'] = result['stdout'].replace(config.probe_ip.address, "[REDACTED]")
            result['stderr'] = result['stderr'].replace(config.probe_ip.address, "[REDACTED]")

        self.report['commands'].append({
            'command_name': ' '.join(command),
            'command_stdout': result['stdout'],
            'command_stderr': result['stderr'],
            'command_exit_reason': result['exit_reason'],
        })
        return result

    def run(self, command, finished=None, env={}, path=None, usePTY=0):
        d = defer.Deferred()
        d.addCallback(self.processEnded, command)
        self.processDirector = ProcessDirector(d, finished, self.timeout)
        self.processDirector.handleRead = self.handleRead
        reactor.spawnProcess(self.processDirector, command[0], command, env=env, path=path, usePTY=usePTY)
        return d

    # handleRead is not an abstract method to be backwards compatible
    def handleRead(self,  stdout,  stderr=None):
        pass



# -*- encoding: utf-8 -*-
#
#  dnsconsistency
#  **************
#
#  The test reports censorship if the cardinality of the intersection of
#  the query result set from the control server and the query result set
#  from the experimental server is zero, which is to say, if the two sets
#  have no matching results whatsoever.
#
#  NOTE: This test frequently results in false positives due to GeoIP-based
#  load balancing on major global sites such as google, facebook, and
#  youtube, etc.
#
# :authors: Arturo Filastò, Isis Lovecruft
# :licence: see LICENSE


from twisted.python import usage
from twisted.internet import defer

from ooni.templates import dnst

from ooni.utils import log


class UsageOptions(usage.Options):
    optParameters = [['backend', 'b', None,
                      'The OONI backend that runs the DNS resolver.'],
                     ['testresolvers', 'T', None,
                      'File containing list of DNS resolvers to test against.'],
                     ['testresolver', 't', None,
                         'Specify a single test resolver to use for testing.']
                     ]


class DNSConsistencyTest(dnst.DNSTest):

    name = "DNS Consistency"
    description = "Checks to see if the DNS responses from a "\
                  "set of DNS resolvers are consistent."
    version = "0.7.0"
    authors = "Arturo Filastò, Isis Lovecruft"

    inputFile = ['file', 'f', None,
                 'Input file of list of hostnames to attempt to resolve']

    requiredTestHelpers = {'backend': 'dns'}
    requiresRoot = False
    requiresTor = False

    usageOptions = UsageOptions
    requiredOptions = ['backend', 'file']

    def setUp(self):
        if (not self.localOptions['testresolvers'] and
                not self.localOptions['testresolver']):
            self.test_resolvers = []
            with open('/etc/resolv.conf') as f:
                for line in f:
                    if line.startswith('nameserver'):
                        self.test_resolvers.append(line.split(' ')[1].strip())
            self.report['test_resolvers'] = self.test_resolvers

        elif self.localOptions['testresolvers']:
            test_resolvers_file = self.localOptions['testresolvers']

        elif self.localOptions['testresolver']:
            self.test_resolvers = [self.localOptions['testresolver']]

        try:
            with open(test_resolvers_file) as f:
                self.test_resolvers = [
                    x.split('#')[0].strip() for x in f.readlines()]
                self.report['test_resolvers'] = self.test_resolvers
            f.close()

        except IOError as e:
            log.exception(e)
            raise usage.UsageError("Invalid test resolvers file")

        except NameError:
            log.debug("No test resolver file configured")

        dns_ip, dns_port = self.localOptions['backend'].split(':')
        self.control_dns_server = (str(dns_ip), int(dns_port))

        self.report['control_resolver'] = "%s:%d" % self.control_dns_server

    @defer.inlineCallbacks
    def test_a_lookup(self):
        """
        We perform an A lookup on the DNS test servers for the domains to be
        tested and an A lookup on the known good DNS server.

        We then compare the results from test_resolvers and that from
        control_resolver and see if they match up.
        If they match up then no censorship is happening (tampering: false).

        If they do not we do a reverse lookup (PTR) on the test_resolvers and
        the control resolver for every IP address we got back and check to see
        if anyone of them matches the control ones.

        If they do, then we take note of the fact that censorship is probably
        not happening (tampering: reverse-match).

        If they do not match then censorship is probably going on (tampering:
        true).
        """
        log.msg("Doing the test lookups on %s" % self.input)
        hostname = self.input

        self.report['successful'] = []
        self.report['failures'] = []
        self.report['inconsistent'] = []

        self.report['errors'] = {}

        try:
            control_answers = yield self.performALookup(hostname,
                                                        self.control_dns_server)

            if not control_answers:
                log.err(
                    "Got no response from control DNS server %s:%d, "
                    "perhaps the DNS resolver is down?" %
                    self.control_dns_server)
                self.report['errors'][
                    "%s:%d" %
                    self.control_dns_server] = 'no_answer'
        except:
            self.report['errors'][
                "%s:%d" %
                self.control_dns_server] = 'error'
            control_answers = None

        for test_resolver in self.test_resolvers:
            log.msg("Testing resolver: %s" % test_resolver)
            test_dns_server = (test_resolver, 53)

            try:
                experiment_answers = yield self.performALookup(hostname,
                                                               test_dns_server)
            except Exception:
                log.err("Problem performing the DNS lookup")
                self.report['errors'][test_resolver] = 'dns_lookup_error'
                self.report['failures'].append(test_resolver)
                continue

            if not experiment_answers:
                log.err("Got no response, perhaps the DNS resolver is down?")
                self.report['errors'][test_resolver] = 'no_answer'
                self.report['failures'].append(test_resolver)
                continue
            else:
                log.debug(
                    "Got the following A lookup answers %s from %s" %
                    (experiment_answers, test_resolver))

            def lookup_details():
                """
                A closure useful for printing test details.
                """
                log.msg("test resolver: %s" % test_resolver)
                log.msg("experiment answers: %s" % experiment_answers)
                log.msg("control answers: %s" % control_answers)

            log.debug(
                "Comparing %s with %s" %
                (experiment_answers, control_answers))

            if not control_answers:
                log.msg("Skipping control resolver comparison")
                self.report['errors'][test_resolver] = None

            elif set(experiment_answers) & set(control_answers):
                lookup_details()
                log.msg("tampering: false")
                self.report['errors'][test_resolver] = False
                self.report['successful'].append(test_resolver)
            else:
                log.msg("Trying to do reverse lookup")
                experiment_reverse = yield self.performPTRLookup(experiment_answers[0],
                                                                 test_dns_server)
                control_reverse = yield self.performPTRLookup(control_answers[0],
                                                              self.control_dns_server)

                if experiment_reverse == control_reverse:
                    log.msg("Further testing has eliminated false positives")
                    lookup_details()
                    log.msg("tampering: reverse_match")
                    self.report['errors'][test_resolver] = 'reverse_match'
                    self.report['successful'].append(test_resolver)
                else:
                    log.msg("Reverse lookups do not match")
                    lookup_details()
                    log.msg("tampering: true")
                    self.report['errors'][test_resolver] = True
                    self.report['inconsistent'].append(test_resolver)

    def inputProcessor(self, filename=None):
        """
        This inputProcessor extracts domain names from urls
        """
        log.debug("Running dnsconsistency default processor")
        if filename:
            fp = open(filename)
            for x in fp.readlines():
                yield x.strip().split('//')[-1].split('/')[0]
            fp.close()
        else:
            pass



from twisted.application import service, internet

from ooni.settings import config
from ooni.api.spec import oonidApplication
from ooni.director import Director

def getOonid():
    director = Director()
    director.start()
    oonidApplication.director = director
    return internet.TCPServer(int(config.advanced.oonid_api_port), oonidApplication)

application = service.Application("ooniprobe")
service = getOonid()
service.setServiceParent(application)



import os
import sys
import codecs
import logging
import traceback

from twisted.python import log as txlog
from twisted.python import util
from twisted.python.failure import Failure
from twisted.python.logfile import DailyLogFile

from ooni import otime

# Get rid of the annoying "No route found for
# IPv6 destination warnings":
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)


def log_encode(logmsg):
    """
    I encode logmsg (a str or unicode) as printable ASCII. Each case
    gets a distinct prefix, so that people differentiate a unicode
    from a utf-8-encoded-byte-string or binary gunk that would
    otherwise result in the same final output.
    """
    if isinstance(logmsg, unicode):
        return codecs.encode(logmsg, 'unicode_escape')
    elif isinstance(logmsg, str):
        try:
            unicodelogmsg = logmsg.decode('utf-8')
        except UnicodeDecodeError:
            return codecs.encode(logmsg, 'string_escape')
        else:
            return codecs.encode(unicodelogmsg, 'unicode_escape')
    else:
        raise Exception("I accept only a unicode object or a string, "
                        "not a %s object like %r" % (type(logmsg),
                                                     repr(logmsg)))


class LogWithNoPrefix(txlog.FileLogObserver):
    def emit(self, eventDict):
        text = txlog.textFromEventDict(eventDict)
        if text is None:
            return

        util.untilConcludes(self.write, "%s\n" % text)
        util.untilConcludes(self.flush)  # Hoorj!


class OONILogger(object):
    def start(self, logfile=None, application_name="ooniprobe"):
        from ooni.settings import config

        if not logfile:
            logfile = os.path.expanduser(config.basic.logfile)

        log_folder = os.path.dirname(logfile)
        log_filename = os.path.basename(logfile)

        daily_logfile = DailyLogFile(log_filename, log_folder)

        txlog.msg("Starting %s on %s (%s UTC)" % (application_name,
                                                  otime.prettyDateNow(),
                                                  otime.prettyDateNowUTC()))

        self.fileObserver = txlog.FileLogObserver(daily_logfile)
        self.stdoutObserver = LogWithNoPrefix(sys.stdout)

        txlog.startLoggingWithObserver(self.stdoutObserver.emit)
        txlog.addObserver(self.fileObserver.emit)

    def stop(self):
        self.stdoutObserver.stop()
        self.fileObserver.stop()

oonilogger = OONILogger()


def start(logfile=None, application_name="ooniprobe"):
    oonilogger.start(logfile, application_name)


def stop():
    oonilogger.stop()


def msg(msg, *arg, **kw):
    from ooni.settings import config
    if config.logging:
        print "%s" % log_encode(msg)


def debug(msg, *arg, **kw):
    from ooni.settings import config
    if config.advanced.debug and config.logging:
        print "[D] %s" % log_encode(msg)


def err(msg, *arg, **kw):
    from ooni.settings import config
    if config.logging:
        if isinstance(msg, Exception):
            msg = "%s: %s" % (msg.__class__.__name__, msg)
        print "[!] %s" % log_encode(msg)


def exception(error):
    """
    Error can either be an error message to print to stdout and to the logfile
    or it can be a twisted.python.failure.Failure instance.
    """
    if isinstance(error, Failure):
        error.printTraceback(sys.stdout)
    else:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        traceback.print_exception(exc_type, exc_value, exc_traceback)



import itertools
from copy import copy

from twisted.web.http_headers import Headers
from twisted.web import error

from twisted.web.client import BrowserLikeRedirectAgent
from twisted.web._newclient import ResponseFailed
from twisted.web._newclient import HTTPClientParser, ParseError
from twisted.python.failure import Failure

from twisted.web import client, _newclient

from twisted.web._newclient import RequestNotSent, RequestGenerationFailed
from twisted.web._newclient import TransportProxyProducer, STATUS

from twisted.internet import reactor
from twisted.internet.defer import Deferred, fail, maybeDeferred, failure

from twisted.python import log

class TrueHeaders(Headers):
    def __init__(self, rawHeaders=None):
        self._rawHeaders = dict()
        if rawHeaders is not None:
            for name, values in rawHeaders.iteritems():
                if type(values) is list:
                    self.setRawHeaders(name, values[:])
                elif type(values) is str:
                    self.setRawHeaders(name, values)

    def setRawHeaders(self, name, values):
        if name.lower() not in self._rawHeaders:
            self._rawHeaders[name.lower()] = dict()
        self._rawHeaders[name.lower()]['name'] = name
        self._rawHeaders[name.lower()]['values'] = values

    def getAllRawHeaders(self):
        for _, v in self._rawHeaders.iteritems():
            yield v['name'], v['values']

    def getRawHeaders(self, name, default=None):
        if name.lower() in self._rawHeaders:
            return self._rawHeaders[name.lower()]['values']
        return default


    def getDiff(self, headers, ignore=[]):
        """

        Args:

            headers: a TrueHeaders object

            ignore: specify a list of header fields to ignore

        Returns:

            a set containing the header names that are not present in
            header_dict or not present in self.
        """
        diff = set()
        field_names = []

        headers_a = copy(self)
        headers_b = copy(headers)
        for name in ignore:
            try:
                del headers_a._rawHeaders[name.lower()]
            except KeyError:
                pass
            try:
                del headers_b._rawHeaders[name.lower()]
            except KeyError:
                pass

        for k, v in itertools.chain(headers_a.getAllRawHeaders(),
                                    headers_b.getAllRawHeaders()):
            field_names.append(k)

        for name in field_names:
            if self.getRawHeaders(name) and headers.getRawHeaders(name):
                pass
            else:
                diff.add(name)
        return list(diff)

class HTTPClientParser(_newclient.HTTPClientParser):
    def logPrefix(self):
        return 'HTTPClientParser'

    def connectionMade(self):
        self.headers = TrueHeaders()
        self.connHeaders = TrueHeaders()
        self.state = STATUS
        self._partialHeader = None

    def headerReceived(self, name, value):
        if self.isConnectionControlHeader(name.lower()):
            headers = self.connHeaders
        else:
            headers = self.headers
        headers.addRawHeader(name, value)

    def statusReceived(self, status):
        # This is a fix for invalid number of parts
        try:
            return _newclient.HTTPClientParser.statusReceived(self, status)
        except ParseError as exc:
            if exc.args[0] == 'wrong number of parts':
                return _newclient.HTTPClientParser.statusReceived(self,
                                                                  status + " XXX")
            raise

class HTTP11ClientProtocol(_newclient.HTTP11ClientProtocol):
    def request(self, request):
        if self._state != 'QUIESCENT':
            return fail(RequestNotSent())

        self._state = 'TRANSMITTING'
        _requestDeferred = maybeDeferred(request.writeTo, self.transport)
        self._finishedRequest = Deferred()

        self._currentRequest = request

        self._transportProxy = TransportProxyProducer(self.transport)
        self._parser = HTTPClientParser(request, self._finishResponse)
        self._parser.makeConnection(self._transportProxy)
        self._responseDeferred = self._parser._responseDeferred

        def cbRequestWrotten(ignored):
            if self._state == 'TRANSMITTING':
                self._state = 'WAITING'
                self._responseDeferred.chainDeferred(self._finishedRequest)

        def ebRequestWriting(err):
            if self._state == 'TRANSMITTING':
                self._state = 'GENERATION_FAILED'
                self.transport.loseConnection()
                self._finishedRequest.errback(
                    failure.Failure(RequestGenerationFailed([err])))
            else:
                log.err(err, 'Error writing request, but not in valid state '
                             'to finalize request: %s' % self._state)

        _requestDeferred.addCallbacks(cbRequestWrotten, ebRequestWriting)

        return self._finishedRequest


class _HTTP11ClientFactory(client._HTTP11ClientFactory):
    noisy = False

    def buildProtocol(self, addr):
        return HTTP11ClientProtocol(self._quiescentCallback)


class HTTPConnectionPool(client.HTTPConnectionPool):
    _factory = _HTTP11ClientFactory

class TrueHeadersAgent(client.Agent):
    def __init__(self, *args, **kw):
        super(TrueHeadersAgent, self).__init__(*args, **kw)
        self._pool = HTTPConnectionPool(reactor, False)

class FixedRedirectAgent(BrowserLikeRedirectAgent):
    """
    This is a redirect agent with this patch manually applied:
    https://twistedmatrix.com/trac/ticket/8265
    """
    def _handleRedirect(self, response, method, uri, headers, redirectCount):
        """
        Handle a redirect response, checking the number of redirects already
        followed, and extracting the location header fields.

        This is patched to fix a bug in infinite redirect loop.
        """
        if redirectCount >= self._redirectLimit:
            err = error.InfiniteRedirection(
                response.code,
                b'Infinite redirection detected',
                location=uri)
            raise ResponseFailed([Failure(err)], response)
        locationHeaders = response.headers.getRawHeaders(b'location', [])
        if not locationHeaders:
            err = error.RedirectWithNoLocation(
                response.code, b'No location header field', uri)
            raise ResponseFailed([Failure(err)], response)
        location = self._resolveLocation(
            # This is the fix to properly handle redirects
            response.request.absoluteURI,
            locationHeaders[0]
        )
        deferred = self._agent.request(method, location, headers)

        def _chainResponse(newResponse):
            newResponse.setPreviousResponse(response)
            return newResponse

        deferred.addCallback(_chainResponse)
        return deferred.addCallback(
            self._handleResponse, method, uri, headers, redirectCount + 1)



"""
This modules contains functionality that is shared amongst ooni-probe and
ooni-backend. If the code in here starts growing too much I think it would
make sense to either:

 * Make the code in here into it's own package that is imported by
 ooni-probe and ooni-backend.

 * Merge ooniprobe with oonibackend.
"""



from __future__ import absolute_import
import re
import os
import random

from hashlib import sha256

from twisted.web import client, http_headers

client._HTTP11ClientFactory.noisy = False

from twisted.internet import reactor, defer

from ooni.utils import log
from ooni import errors

try:
    from pygeoip import GeoIP
except ImportError:
    try:
        import GeoIP as CGeoIP
        def GeoIP(database_path, *args, **kwargs):
            return CGeoIP.open(database_path, CGeoIP.GEOIP_STANDARD)
    except ImportError:
        log.err("Unable to import pygeoip. We will not be able to run geo IP related measurements")

class GeoIPDataFilesNotFound(Exception):
    pass

def IPToLocation(ipaddr):
    from ooni.settings import config

    country_file = config.get_data_file_path('GeoIP/GeoIP.dat')
    asn_file = config.get_data_file_path('GeoIP/GeoIPASNum.dat')

    location = {'city': None, 'countrycode': 'ZZ', 'asn': 'AS0'}
    if not asn_file or not country_file:
        log.err("Could not find GeoIP data file in data directories."
                "Try running ooniresources or"
                " edit your ooniprobe.conf")
        return location

    country_dat = GeoIP(country_file)
    asn_dat = GeoIP(asn_file)

    country_code = country_dat.country_code_by_addr(ipaddr)
    if country_code is not None:
        location['countrycode'] =  country_code

    asn = asn_dat.org_by_addr(ipaddr)
    if asn is not None:
        location['asn'] = asn.split(' ')[0]

    return location

def database_version():
    from ooni.settings import config

    version = {
        'GeoIP': {
            'sha256': None,
            'timestamp': None,
        },
        'GeoIPASNum': {
            'sha256': None,
            'timestamp': None
        }
    }

    for key in version.keys():
        geoip_file = config.get_data_file_path("GeoIP/" + key + ".dat")
        if not geoip_file or not os.path.isfile(geoip_file):
            continue
        timestamp = os.stat(geoip_file).st_mtime

        sha256hash = sha256()
        with open(geoip_file) as f:
            while True:
                chunk = f.read(8192)
                if not chunk:
                    break
                sha256hash.update(chunk)

        version[key]['timestamp'] = timestamp
        version[key]['sha256'] = sha256hash.hexdigest()
    return version


class HTTPGeoIPLookupper(object):
    url = None

    _agent = client.Agent

    def __init__(self):
        self.agent = self._agent(reactor)

    def _response(self, response):
        from ooni.utils.net import BodyReceiver

        content_length = response.headers.getRawHeaders('content-length')

        finished = defer.Deferred()
        response.deliverBody(BodyReceiver(finished, content_length))
        finished.addCallback(self.parseResponse)
        return finished

    def parseResponse(self, response_body):
        """
        Override this with the logic for parsing the response.

        Should return the IP address of the probe.
        """
        pass

    def failed(self, failure):
        log.err("Failed to lookup via %s" % self.url)
        log.exception(failure)
        return failure

    def lookup(self):
        from ooni.utils.net import userAgents

        headers = {}
        headers['User-Agent'] = [random.choice(userAgents)]

        d = self.agent.request("GET", self.url, http_headers.Headers(headers))
        d.addCallback(self._response)
        d.addErrback(self.failed)
        return d

class UbuntuGeoIP(HTTPGeoIPLookupper):
    url = "http://geoip.ubuntu.com/lookup"

    def parseResponse(self, response_body):
        m = re.match(".*<Ip>(.*)</Ip>.*", response_body)
        probe_ip = m.group(1)
        return probe_ip

class TorProjectGeoIP(HTTPGeoIPLookupper):
    url = "https://check.torproject.org/"

    def parseResponse(self, response_body):
        regexp = "Your IP address appears to be:  <strong>((\d+\.)+(\d+))"
        probe_ip = re.search(regexp, response_body).group(1)
        return probe_ip

class ProbeIP(object):
    strategy = None
    address = None

    def __init__(self):
        self.geoIPServices = {
            'ubuntu': UbuntuGeoIP,
            'torproject': TorProjectGeoIP
        }
        self.geodata = {
            'asn': 'AS0',
            'city': None,
            'countrycode': 'ZZ',
            'ip': '127.0.0.1'
        }

    def resolveGeodata(self):
        from ooni.settings import config

        self.geodata = IPToLocation(self.address)
        self.geodata['ip'] = self.address
        if not config.privacy.includeasn:
            self.geodata['asn'] = 'AS0'
        if not config.privacy.includecountry:
            self.geodata['countrycode'] = 'ZZ'
        if not config.privacy.includeip:
            self.geodata['ip'] = '127.0.0.1'

    @defer.inlineCallbacks
    def lookup(self):
        if self.address:
            defer.returnValue(self.address)
        else:
            try:
                yield self.askTor()
                log.msg("Found your IP via Tor")
                self.resolveGeodata()
                defer.returnValue(self.address)
            except errors.TorStateNotFound:
                log.debug("Tor is not running. Skipping IP lookup via Tor.")
            except Exception:
                log.msg("Unable to lookup the probe IP via Tor.")

            try:
                yield self.askGeoIPService()
                log.msg("Found your IP via a GeoIP service")
                self.resolveGeodata()
                defer.returnValue(self.address)
            except Exception:
                log.msg("Unable to lookup the probe IP via GeoIPService")
                raise

    @defer.inlineCallbacks
    def askGeoIPService(self):
        # Shuffle the order in which we test the geoip services.
        services = self.geoIPServices.items()
        random.shuffle(services)
        for service_name, service in services:
            s = service()
            log.msg("Looking up your IP address via %s" % service_name)
            try:
                self.address = yield s.lookup()
                self.strategy = 'geo_ip_service-' + service_name
                break
            except Exception:
                log.msg("Failed to lookup your IP via %s" % service_name)

        if not self.address:
            raise errors.ProbeIPUnknown

    def askTor(self):
        """
        Obtain the probes IP address by asking the Tor Control port via GET INFO
        address.

        XXX this lookup method is currently broken when there are cached descriptors or consensus documents
        see: https://trac.torproject.org/projects/tor/ticket/8214
        """
        from ooni.settings import config

        if config.tor_state:
            d = config.tor_state.protocol.get_info("address")
            @d.addCallback
            def cb(result):
                self.strategy = 'tor_get_info_address'
                self.address = result.values()[0]
            return d
        else:
            raise errors.TorStateNotFound



# -*- encoding: utf-8 -*-
#
# :authors: Arturo Filastò
# :licence: see LICENSE

import random
from twisted.python import usage, failure

from ooni.utils import log
from ooni.utils.net import userAgents
from ooni.templates import httpt
from ooni.errors import failureToString

class MissingInput(Exception):
    pass

class UsageOptions(usage.Options):
    optParameters = [
        ['url', 'u', None, 'Specify a single URL to test.'],
        ['factor', 'f', 0.8,
         'What factor should be used for triggering censorship '
         '(0.8 == 80%).']]
    optFlags = [
        ["withoutbody", "B", "don't include HTTP response body inside of the "
         "report."],
        ]


class HTTPRequestsTest(httpt.HTTPTest):

    """
    Performs a two GET requests to the set of sites to be tested for
    censorship, one over a known good control channel (Tor), the other over the
    test network.

    We check to see if the response headers match and if the response body
    lengths match.
    """
    name = "HTTP Requests"
    description = ("Performs a HTTP GET request over Tor and one over the "
                  "local network and compares the two results.")
    author = "Arturo Filastò"
    version = "0.2.5"

    usageOptions = UsageOptions

    inputFile = ['file', 'f', None,
                 'List of URLS to perform GET and POST requests to']
    requiresRoot = False
    requiresTor = True

    # These values are used for determining censorship based on response body
    # lengths
    control_body_length = None
    experiment_body_length = None

    def requirements(self):
        if not self.localOptions['url'] and \
                not self.localOptions['file']:
            raise MissingInput("You did not specify either a URL with -u "
                               "or an input file with -f")

    def setUp(self):
        """
        Check for inputs.
        """
        if self.input:
            self.url = self.input
        elif self.localOptions['url']:
            self.url = self.localOptions['url']
        else:
            raise Exception("No input specified")
        self.report['input'] = self.url

        self.factor = self.localOptions['factor']
        self.report['control_failure'] = None
        self.report['experiment_failure'] = None
        self.report['body_length_match'] = None
        self.report['body_proportion'] = None
        self.report['factor'] = float(self.factor)
        self.report['headers_diff'] = None
        self.report['headers_match'] = None
        self.report['control_cloudflare'] = None

        self.headers = {'User-Agent': [random.choice(userAgents)]}

    def compare_body_lengths(self, body_length_a, body_length_b):

        if body_length_b == 0 and body_length_a != 0:
            rel = float(body_length_b)/float(body_length_a)
        elif body_length_b == 0 and body_length_a == 0:
            rel = float(1)
        else:
            rel = float(body_length_a)/float(body_length_b)

        if rel > 1:
            rel = 1/rel

        self.report['body_proportion'] = rel
        if rel > float(self.factor):
            log.msg("The two body lengths appear to match")
            log.msg("censorship is probably not happening")
            self.report['body_length_match'] = True
        else:
            log.msg("The two body lengths appear to not match")
            log.msg("censorship could be happening")
            self.report['body_length_match'] = False

    def compare_headers(self, headers_a, headers_b):
        diff = headers_a.getDiff(headers_b)
        if diff:
            log.msg("Headers appear to *not* match")
            self.report['headers_diff'] = diff
            self.report['headers_match'] = False
        else:
            log.msg("Headers appear to match")
            self.report['headers_diff'] = diff
            self.report['headers_match'] = True

    def test_get_experiment(self):
        log.msg("Performing GET request to %s" % self.url)
        return self.doRequest(self.url, method="GET",
                              use_tor=False, headers=self.headers)

    def test_get_control(self):
        log.msg("Performing GET request to %s over Tor" % self.url)
        return self.doRequest(self.url, method="GET",
                              use_tor=True, headers=self.headers)

    def postProcessor(self, measurements):
        experiment = control = None
        for status, measurement in measurements:
            net_test_method = measurement.netTestMethod.im_func.func_name

            if net_test_method == "test_get_experiment":
                if isinstance(measurement.result, failure.Failure):
                    self.report['experiment_failure'] = failureToString(
                        measurement.result)
                else:
                    experiment = measurement.result
            elif net_test_method == "test_get_control":
                if isinstance(measurement.result, failure.Failure):
                    self.report['control_failure'] = failureToString(
                        measurement.result)
                else:
                    control = measurement.result

        if experiment and control:
            if hasattr(experiment, 'body') and hasattr(control, 'body') \
                    and experiment.body and control.body:
                self.report['control_cloudflare'] = False
                if 'Attention Required! | CloudFlare' in control.body:
                    log.msg("The control body contains a blockpage from "
                            "cloudflare. This will skew our results.")
                    self.report['control_cloudflare'] = True
                self.compare_body_lengths(len(control.body),
                                          len(experiment.body))
            if hasattr(experiment, 'headers') and hasattr(control, 'headers') \
                    and experiment.headers and control.headers:
                self.compare_headers(control.headers,
                                     experiment.headers)
        return self.report



# -*- encoding: utf-8 -*-
#
# Squid transparent HTTP proxy detector
# *************************************
#
# :authors: Arturo Filastò
# :licence: see LICENSE

from ooni import utils
from ooni.utils import log
from ooni.templates import httpt
import re

class SquidTest(httpt.HTTPTest):
    """
    This test aims at detecting the presence of a squid based transparent HTTP
    proxy. It also tries to detect the version number.
    """
    name = "Squid test"
    author = "Arturo Filastò"
    version = "0.1"

    optParameters = [['backend', 'b', 'http://ooni.nu/test/', 'Test backend to use']]

    #inputFile = ['urls', 'f', None, 'Urls file']
    inputs =['http://google.com']

    requiresRoot = False
    requiresTor = False

    def test_cacheobject(self):
        """
        This detects the presence of a squid transparent HTTP proxy by sending
        a request for cache_object://localhost/info.

        The response to this request will usually also contain the squid
        version number.
        """
        log.debug("Running")
        def process_body(body):
            if "Access Denied." in body:
                self.report['transparent_http_proxy'] = True
            else:
                self.report['transparent_http_proxy'] = False

        log.msg("Testing Squid proxy presence by sending a request for "\
                "cache_object")
        headers = {}
        #headers["Host"] = [self.input]
        self.report['trans_http_proxy'] = None
        method = "GET"
        body = "cache_object://localhost/info"
        return self.doRequest(self.localOptions['backend'], method=method, body=body,
                        headers=headers, body_processor=process_body)

    def test_search_bad_request(self):
        """
        Attempts to perform a request with a random invalid HTTP method.

        If we are being MITMed by a Transparent Squid HTTP proxy we will get
        back a response containing the X-Squid-Error header.
        """
        def process_headers(headers):
            log.debug("Processing headers in test_search_bad_request")
            if 'X-Squid-Error' in headers:
                log.msg("Detected the presence of a transparent HTTP "\
                        "squid proxy")
                self.report['trans_http_proxy'] = True
            else:
                log.msg("Did not detect the presence of transparent HTTP "\
                        "squid proxy")
                self.report['transparent_http_proxy'] = False

        log.msg("Testing Squid proxy presence by sending a random bad request")
        headers = {}
        #headers["Host"] = [self.input]
        method = utils.randomSTR(10, True)
        self.report['transparent_http_proxy'] = None
        return self.doRequest(self.localOptions['backend'], method=method,
                        headers=headers, headers_processor=process_headers)

    def test_squid_headers(self):
        """
        Detects the presence of a squid transparent HTTP proxy based on the
        response headers it adds to the responses to requests.
        """
        def process_headers(headers):
            """
            Checks if any of the headers that squid is known to add match the
            squid regexp.

            We are looking for something that looks like this:

                via: 1.0 cache_server:3128 (squid/2.6.STABLE21)
                x-cache: MISS from cache_server
                x-cache-lookup: MISS from cache_server:3128
            """
            squid_headers = {'via': r'.* \((squid.*)\)',
                        'x-cache': r'MISS from (\w+)',
                        'x-cache-lookup': r'MISS from (\w+:?\d+?)'
                        }

            self.report['transparent_http_proxy'] = False
            for key in squid_headers.keys():
                if key in headers:
                    log.debug("Found %s in headers" % key)
                    m = re.search(squid_headers[key], headers[key])
                    if m:
                        log.msg("Detected the presence of squid transparent"\
                                " HTTP Proxy")
                        self.report['transparent_http_proxy'] = True

        log.msg("Testing Squid proxy by looking at response headers")
        headers = {}
        #headers["Host"] = [self.input]
        method = "GET"
        self.report['transparent_http_proxy'] = None
        d = self.doRequest(self.localOptions['backend'], method=method,
                        headers=headers, headers_processor=process_headers)
        return d





import os
import csv
import GeoIP

from ooni.settings import config


class GeoIPDB(object):
    _borg = {}
    country = None

    def __init__(self):
        self.__dict__ = self._borg
        if not self.country:
            try:
                country_file = config.get_data_file_path('GeoIP/GeoIP.dat')
                self.country = GeoIP.open(country_file,
                                          GeoIP.GEOIP_STANDARD)
            except:
                raise Exception("Edit the geoip_data_dir line in your config"
                                " file to point to your geoip files")


def generate_country_input(country_code, dst):

    csv_file = config.get_data_file_path("resources/"
                                         "namebench-dns-servers.csv")

    filename = os.path.join(dst, "dns-server-%s.txt" % country_code)
    fw = open(filename, "w")
    geoip_db = GeoIPDB()
    reader = csv.reader(open(csv_file))
    for row in reader:
        if row[2] == 'X-Internal-IP':
            continue
        elif row[2] == 'X-Unroutable':
            continue
        elif row[2] == 'X-Link_local':
            continue
        ipaddr = row[0]
        cc = geoip_db.country.country_code_by_addr(ipaddr)
        if not cc:
            continue
        if cc.lower() == country_code.lower():
            fw.write(ipaddr + "\n")
    fw.close()
    return filename


def generate_global_input(dst):
    pass



from twisted.internet import reactor
from ooni.common.txextra import HTTPConnectionPool

from twisted import version as twisted_version
from twisted.python.versions import Version
_twisted_15_0 = Version('twisted', 15, 0, 0)

from txsocksx.http import SOCKS5Agent
from txsocksx.client import SOCKS5ClientFactory

SOCKS5ClientFactory.noisy = False

class TrueHeadersSOCKS5Agent(SOCKS5Agent):
    def __init__(self, *args, **kw):
        super(TrueHeadersSOCKS5Agent, self).__init__(*args, **kw)
        pool = HTTPConnectionPool(reactor, False)
        #
        # With Twisted > 15.0 txsocksx wraps the twisted agent using a
        # wrapper class, hence we must set the _pool attribute in the
        # inner class rather than into its external wrapper.
        #
        if twisted_version >= _twisted_15_0:
            self._wrappedAgent._pool = pool
        else:
            self._pool = pool



# -*- encoding: utf-8 -*-
import yaml

from twisted.python import usage

from ooni.templates import httpt
from ooni.utils import log

class UsageOptions(usage.Options):
    """
    See https://github.com/hellais/ooni-inputs/processed/uk_mobile_networks_redirects.yaml 
    to see how the rules file should look like.
    """
    optParameters = [
                     ['rules', 'y', None, 
                    'Specify the redirect rules file ']
                    ]

class HTTPUKMobileNetworksTest(httpt.HTTPTest):
    """
    This test was thought of by Open Rights Group and implemented with the
    purpose of detecting censorship in the UK.
    For more details on this test see:
    https://trac.torproject.org/projects/tor/ticket/6437
    XXX port the knowledge from the trac ticket into this test docstring
    """
    name = "HTTP UK mobile network redirect test"

    usageOptions = UsageOptions

    followRedirects = True

    inputFile = ['urls', 'f', None, 'List of urls one per line to test for censorship']
    requiredOptions = ['urls']
    requiresRoot = False
    requiresTor = False

    def testPattern(self, value, pattern, type):
        if type == 'eq':
            return value == pattern
        elif type == 're':
            import re
            if re.match(pattern, value):
                return True
            else:
                return False
        else:
            return None

    def testPatterns(self, patterns, location):
        test_result = False

        if type(patterns) == list:
            for pattern in patterns:
                test_result |= self.testPattern(location, pattern['value'], pattern['type'])
        rules_file = self.localOptions['rules']

        return test_result

    def testRules(self, rules, location):
        result = {}
        blocked = False
        for rule, value in rules.items():
            current_rule = {}
            current_rule['name'] = value['name']
            current_rule['patterns'] = value['patterns']
            current_rule['test'] = self.testPatterns(value['patterns'], location)
            blocked |= current_rule['test']
            result[rule] = current_rule
        result['blocked'] = blocked
        return result

    def processRedirect(self, location):
        self.report['redirect'] = None
        rules_file = self.localOptions['rules']

        fp = open(rules_file)
        rules = yaml.safe_load(fp)
        fp.close()

        log.msg("Testing rules %s" % rules)
        redirect = self.testRules(rules, location)
        self.report['redirect'] = redirect



import random

from twisted.internet import defer, reactor
from twisted.internet.protocol import Protocol, Factory
from scapy.all import get_if_list
import txtorcon

from ooni.settings import OConfig
from ooni import errors
from ooni.utils import net
from bases import ConfigTestCase


class TestSettings(ConfigTestCase):
    def setUp(self):
        super(TestSettings, self).setUp()
        self.conf = OConfig()
        self.configuration = {'advanced': {'interface': 'auto',
                                           'start_tor': True},
                              'tor': {}}
        self.silly_listener = None
        self.tor_protocol = None

    def tearDown(self):
        super(TestSettings, self).tearDown()
        if self.silly_listener is not None:
            self.silly_listener.stopListening()

    def run_tor(self):
        def progress(percent, tag, summary):
            ticks = int((percent/100.0) * 10.0)
            prog = (ticks * '#') + ((10 - ticks) * '.')
            print '%s %s' % (prog, summary)

        config = txtorcon.TorConfig()
        config.SocksPort = self.conf.tor.socks_port
        config.ControlPort = self.conf.tor.control_port
        d = txtorcon.launch_tor(config, reactor, progress_updates=progress)
        return d

    def run_silly_server(self):
        class SillyProtocol(Protocol):
            def __init__(self, factory):
                self.factory = factory

        class SillyFactory(Factory):
            protocol = SillyProtocol

            def buildProtocol(self, address):
                p = self.protocol(self)
                return p

        self.silly_listener = reactor.listenTCP(self.conf.tor.socks_port, SillyFactory())

    def test_vanilla_configuration(self):
        self.conf.check_incoherences(self.configuration)

    @defer.inlineCallbacks
    def test_check_tor_missing_options(self):
        self.conf.advanced.start_tor = False
        try:
            yield self.conf.check_tor()
        except errors.ConfigFileIncoherent:
            pass

        self.conf.tor.socks_port = 9999
        try:
            yield self.conf.check_tor()
        except errors.ConfigFileIncoherent:
            pass

        self.conf.tor.socks_port = None
        self.conf.tor.control_port = 9998
        try:
            yield self.conf.check_tor()
        except errors.ConfigFileIncoherent:
            pass

    @defer.inlineCallbacks
    def test_check_tor_correct(self):
        """
        This test has been disabled because there is a strange concatenation of
        conditions that make it not possible to run it on travis.
        The tests need to be run as root on travis so that the ones that use
        scapy will work properly. When running tor as root, though, it will by
        default drop privileges to a lesser priviledged user (on debian based
        systems debian-tor). The problem is that the datadir will have already
        been created with the privileges of root, hence it will fail to use it
        as a datadir and fail.
        txtorcon addressed this issue in https://github.com/meejah/txtorcon/issues/26
        by chmodding the datadir with what is set as User.
        So we could either:

            1) Set User to root so that tor has access to that directory, but
            this will not work because then it will not be happy that
            /var/run/tor has more lax permissions (also debian-tor can read it)
            so it will fail. We could disable the control port, hence not
            needing to use /var/run/tor, but this is not possible due to:
            https://github.com/meejah/txtorcon/issues/80

            2) We set the User to be the owner of /var/run/tor, but this does
            not exist on all systems, so it would only work for travis.

        For the time being I am just going to disable this test and wait for
        one of the above bugs to have a better fix.
        """
        self.skipTest("See comment in the code")
        self.conf.advanced.start_tor = False
        self.conf.tor.socks_port = net.randomFreePort()
        self.conf.tor.control_port = net.randomFreePort()
        self.tor_process = yield self.run_tor()
        yield self.conf.check_incoherences(self.configuration)
        self.tor_process.transport.signalProcess('TERM')

        d = defer.Deferred()
        reactor.callLater(10, d.callback, None)
        yield d

    @defer.inlineCallbacks
    def test_check_tor_silly_listener(self):
        self.conf.advanced.start_tor = False
        self.conf.tor.socks_port = net.randomFreePort()
        self.conf.tor.control_port = None
        self.run_silly_server()
        try:
            yield self.conf.check_tor()
        except errors.ConfigFileIncoherent:
            pass

    def test_check_incoherences_interface(self):
        self.configuration['advanced']['interface'] = 'funky'
        self.assertRaises(errors.ConfigFileIncoherent, self.conf.check_incoherences, self.configuration)

        self.configuration['advanced']['interface'] = random.choice(get_if_list())
        self.conf.check_incoherences(self.configuration)



from twisted.trial import unittest

import ooni.errors

class TestErrors(unittest.TestCase):

    def test_catch_child_failures_before_parent_failures(self):
        """
        Verify that more specific Failures are caught first by
        handleAllFailures() and failureToString().

        Fails if a subclass is listed after its parent Failure.
        """

        # Check each Failure against all subsequent failures
        for index, (failure, _) in enumerate(ooni.errors.known_failures):
            for sub_failure, _ in ooni.errors.known_failures[index+1:]:

                # Fail if subsequent Failure inherits from the current Failure
                self.assertNotIsInstance(sub_failure(None), failure)
